@@ -21,6 +21,7 @@
 
 const Lang = imports.lang;
 const Gettext = imports.gettext;
+const _ = imports.gettext.gettext;
 
 const GtkClutter = imports.gi.GtkClutter;
 const EvDoc = imports.gi.EvinceDocument;
@@ -47,15 +48,24 @@ const Selections = imports.selections;
 const Sources = imports.sources;
 const TrackerController = imports.trackerController;
 const Tweener = imports.util.tweener;
+const Utils = imports.utils;
 const WindowMode = imports.windowMode;
 
-
 function Application() {
     this._init();
 }
 
 Application.prototype = {
     _init: function() {
+        Gettext.bindtextdomain('gnome-documents', Path.LOCALE_DIR);
+        Gettext.textdomain('gnome-documents');
+        GLib.set_prgname('gnome-documents');
+
+        GLib.setenv('GJS_DEBUG_OUTPUT', 'stderr', true);
+        GLib.setenv('GJS_DEBUG_TOPICS', 'JS ERROR;JS LOG', true);
+
+        Global.settings = new Gio.Settings({ schema: 'org.gnome.documents' });
+
         // TODO: subclass Gtk.Application once we support GObject inheritance,
         //       see https://bugzilla.gnome.org/show_bug.cgi?id=663492
         this.application = new Gtk.Application({
@@ -71,9 +81,52 @@ Application.prototype = {
             }));
     },
 
+    _initMenus: function() {
+	let quitAction = new Gio.SimpleAction({ name: 'quit' });
+	quitAction.connect('activate', Lang.bind(this,
+            function() {
+                this._mainWindow.window.destroy();
+	    }));
+	this.application.add_action(quitAction);
+
+        let fsAction = new Gio.SimpleAction({ name: 'fullscreen' });
+        fsAction.connect('activate', Lang.bind(this,
+            function() {
+                Global.modeController.toggleFullscreen();
+            }));
+        Global.modeController.connect('window-mode-changed', Lang.bind(this,
+            function() {
+                let mode = Global.modeController.getWindowMode();
+                fsAction.set_enabled(mode == WindowMode.WindowMode.PREVIEW);
+            }));
+        this.application.add_action(fsAction);
+
+        let viewAsAction = Gio.SimpleAction.new_stateful('view-as',
+                                                         GLib.VariantType.new('s'),
+                                                         Utils.listSettingToMenu());
+        viewAsAction.connect('activate', Lang.bind(this,
+            function(action, variant) {
+                Global.settings.set_boolean('list-view', Utils.listMenuToSetting(variant));
+            }));
+        Global.settings.connect('changed::list-view', Lang.bind(this,
+            function() {
+                viewAsAction.state = Utils.listSettingToMenu();
+            }));
+        this.application.add_action(viewAsAction);
+
+	let menu = new Gio.Menu();
+        menu.append(_("Fullscreen"), 'app.fullscreen');
+	menu.append(_("Quit"), 'app.quit');
+
+        let viewAs = new Gio.Menu();
+        viewAs.append(_("Grid"), 'app.view-as::grid');
+        viewAs.append(_("List"), 'app.view-as::list');
+        menu.prepend_section(_("View as"), viewAs);
+
+	this.application.set_app_menu(menu);
+    },
+
     _onStartup: function() {
-        Gettext.bindtextdomain('gnome-documents', Path.LOCALE_DIR);
-        Gettext.textdomain('gnome-documents');
         String.prototype.format = Format.format;
 
         GtkClutter.init(null, null);
@@ -81,7 +134,6 @@ Application.prototype = {
         Tweener.init();
 
         Global.application = this;
-        Global.settings = new Gio.Settings({ schema: 'org.gnome.documents' });
         Global.offsetController = new OffsetController.OffsetController();
         Global.searchController = new Searchbar.SearchController();
         Global.errorHandler = new Error.ErrorHandler();
@@ -114,8 +166,8 @@ Application.prototype = {
         Global.selectionController = new Selections.SelectionController();
         Global.modeController = new WindowMode.ModeController();
 
-        this._mainWindow = new MainWindow.MainWindow();
-        this.application.add_window(this._mainWindow.window);
+        this._initMenus();
+        this._mainWindow = new MainWindow.MainWindow(this.application);
     },
 
     _commandLine: function(app, commandLine) {
