@@ -24,6 +24,7 @@ const Gio = imports.gi.Gio;
 const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const GData = imports.gi.GData;
+const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const _ = imports.gettext.gettext;
@@ -31,15 +32,240 @@ const _ = imports.gettext.gettext;
 const Lang = imports.lang;
 const Signals = imports.signals;
 
-const Categories = imports.categories;
 const ChangeMonitor = imports.changeMonitor;
 const Global = imports.global;
+const Manager = imports.manager;
 const Path = imports.path;
 const Query = imports.query;
+const Searchbar = imports.searchbar;
 const TrackerUtils = imports.trackerUtils;
 const Utils = imports.utils;
 
-const _THUMBNAIL_FRAME = 3;
+function SingleItemJob(doc) {
+    this._init(doc);
+}
+
+SingleItemJob.prototype = {
+    _init: function(urn) {
+        this._urn = urn;
+        this._cursor = null;
+    },
+
+    run: function(flags, callback) {
+        this._callback = callback;
+
+        let query = Global.queryBuilder.buildSingleQuery(this._urn, flags);
+        Global.connectionQueue.add(query.sparql, null, Lang.bind(this,
+            function(object, res) {
+                try {
+                    let cursor = object.query_finish(res);
+                    cursor.next_async(null, Lang.bind(this, this._onCursorNext));
+                } catch (e) {
+                    log('Unable to query single item ' + e.toString());
+                    this._emitCallback();
+                }
+            }));
+    },
+
+    _onCursorNext: function(cursor, res) {
+        let valid = false;
+
+        try {
+            valid = cursor.next_finish(res);
+        } catch (e) {
+            log('Unable to query single item ' + e.toString());
+        }
+
+        if (!valid) {
+            cursor.close();
+            this._emitCallback();
+
+            return;
+        }
+
+        this._cursor = cursor;
+        this._emitCallback();
+        cursor.close();
+    },
+
+    _emitCallback: function() {
+        this._callback(this._cursor);
+    }
+};
+
+function DeleteItemJob(urn) {
+    this._init(urn);
+}
+
+// deletes the given resource
+DeleteItemJob.prototype = {
+    _init: function(urn) {
+        this._urn = urn;
+    },
+
+    run: function(callback) {
+        this._callback = callback;
+
+        let query = Global.queryBuilder.buildDeleteResourceQuery(this._urn);
+        Global.connectionQueue.update(query.sparql, null, Lang.bind(this,
+            function(object, res) {
+                try {
+                    object.update_finish(res);
+                } catch (e) {
+                    log(e);
+                }
+
+                this._callback();
+            }));
+    }
+};
+
+function CollectionIconWatcher(collection) {
+    this._init(collection);
+}
+
+CollectionIconWatcher.prototype = {
+    _init: function(collection) {
+        this._collection = collection;
+        this._pixbuf = null;
+
+        this._start();
+    },
+
+    _clear: function() {
+        this._docConnections = {};
+        this._urns = [];
+        this._docs = [];
+    },
+
+    _start: function() {
+        this._clear();
+
+        let query = Global.queryBuilder.buildCollectionIconQuery(this._collection.id);
+        Global.connectionQueue.add(query.sparql, null, Lang.bind(this,
+            function(object, res) {
+                let cursor = null;
+                try {
+                    cursor = object.query_finish(res);
+                } catch (e) {
+                    log('Unable to query collection items ' + e.toString());
+                    return;
+                }
+
+                cursor.next_async(null, Lang.bind(this, this._onCursorNext));
+            }));
+    },
+
+    _onCursorNext: function(cursor, res) {
+        let valid = false;
+
+        try {
+            valid = cursor.next_finish(res);
+        } catch (e) {
+            log('Unable to query collection items ' + e.toString());
+            cursor.close();
+            return;
+        }
+
+        if (!valid) {
+            cursor.close();
+            this._onCollectionIconFinished();
+
+            return;
+        }
+
+        let urn = cursor.get_string(0)[0];
+        this._urns.push(urn);
+
+        cursor.next_async(null, Lang.bind(this, this._onCursorNext));
+    },
+
+    _onCollectionIconFinished: function() {
+        if (!this._urns.length)
+            return;
+
+        // now this._urns has all the URNs of items contained in the collection
+        let toQuery = [];
+
+        this._urns.forEach(Lang.bind(this,
+            function(urn) {
+                let doc = Global.documentManager.getItemById(urn);
+                if (doc)
+                    this._docs.push(doc);
+                else
+                    toQuery.push(urn);
+            }));
+
+        this._toQueryRemaining = toQuery.length;
+        if (!this._toQueryRemaining) {
+            this._allDocsReady();
+            return;
+        }
+
+        toQuery.forEach(Lang.bind(this,
+            function(urn) {
+                let job = new SingleItemJob(urn);
+                job.run(Query.QueryFlags.UNFILTERED, Lang.bind(this,
+                    function(cursor) {
+                        if (cursor) {
+                            let doc = Global.documentManager.createDocumentFromCursor(cursor);
+                            this._docs.push(doc);
+                        }
+
+                        this._toQueryCollector();
+                    }));
+            }));
+    },
+
+    _toQueryCollector: function() {
+        this._toQueryRemaining--;
+
+        if (!this._toQueryRemaining)
+            this._allDocsReady();
+    },
+
+    _allDocsReady: function() {
+        this._docs.forEach(Lang.bind(this,
+            function(doc) {
+                let updateId = doc.connect('info-updated',
+                                           Lang.bind(this, this._createCollectionIcon));
+                this._docConnections[updateId] = doc;
+            }));
+
+        this._createCollectionIcon();
+    },
+
+    _createCollectionIcon: function() {
+        // now this._docs has an array of Document objects from which we will create the
+        // collection icon
+        let pixbufs = [];
+
+        this._docs.forEach(
+            function(doc) {
+                pixbufs.push(doc.pristinePixbuf);
+            });
+
+        this._pixbuf = Gd.create_collection_icon(Utils.getIconSize(), pixbufs);
+        this._emitRefresh();
+    },
+
+    _emitRefresh: function() {
+        this.emit('icon-updated', this._pixbuf);
+    },
+
+    destroy: function() {
+        for (id in this._docConnections) {
+            let doc = this._docConnections[id];
+            doc.disconnect(id);
+        }
+    },
+
+    refresh: function() {
+        this.destroy();
+        this._start();
+    }
+};
+Signals.addSignalMethods(CollectionIconWatcher.prototype);
 
 function DocCommon(cursor) {
     this._init(cursor);
@@ -47,14 +273,15 @@ function DocCommon(cursor) {
 
 DocCommon.prototype = {
     _init: function(cursor) {
-        this.urn = null;
+        this.id = null;
         this.uri = null;
-        this.title = null;
+        this.name = null;
         this.author = null;
         this.mtime = null;
         this.resourceUrn = null;
         this.favorite = null;
         this.pixbuf = null;
+        this.pristinePixbuf = null;
         this.defaultAppName = null;
 
         this.mimeType = null;
@@ -65,49 +292,41 @@ DocCommon.prototype = {
         this.favorite = false;
         this.shared = false;
 
-        this._thumbnailed = false;
+        this.collection = false;
+        this._collectionIconWatcher = null;
+
+        this.thumbnailed = false;
+        this._thumbPath = null;
 
         this.populateFromCursor(cursor);
 
         this._refreshIconId =
             Global.settings.connect('changed::list-view',
                                     Lang.bind(this, this.refreshIcon));
-        this._categoryId =
-            Global.categoryManager.connect('active-category-changed',
-                                           Lang.bind(this, this.refreshIcon));
+        this._filterId =
+            Global.searchCategoryManager.connect('active-changed',
+                                                 Lang.bind(this, this.refreshIcon));
     },
 
     refresh: function() {
-        let sparql = Global.queryBuilder.buildSingleQuery(this.urn);
-
-        Global.connection.query_async(sparql, null, Lang.bind(this,
-            function(object, res) {
-                let cursor = null;
-
-                try {
-                    cursor = object.query_finish(res);
-                    cursor.next_async(null, Lang.bind(this,
-                        function(object, res) {
-                            let valid = object.next_finish(res);
-                            if (valid)
-                                this.populateFromCursor(object);
-
-                            cursor.close();
-                        }));
-                } catch (e) {
-                    log('Unable to refresh file information: ' + e.toString());
+        let job = new SingleItemJob(this.id);
+        job.run(Query.QueryFlags.NONE, Lang.bind(this,
+            function(cursor) {
+                if (!cursor)
                     return;
-                }
+
+                this.populateFromCursor(cursor);
             }));
     },
 
     _sanitizeTitle: function() {
-        this.title = this.title.replace('Microsoft Word - ', '', 'g');
+        this.name = this.name.replace('Microsoft Word - ', '', 'g');
     },
 
     populateFromCursor: function(cursor) {
         this.uri = cursor.get_string(Query.QueryColumns.URI)[0];
-        this.urn = cursor.get_string(Query.QueryColumns.URN)[0];
+        this.id = cursor.get_string(Query.QueryColumns.URN)[0];
+        this.identifier = cursor.get_string(Query.QueryColumns.IDENTIFIER)[0];
         this.author = cursor.get_string(Query.QueryColumns.AUTHOR)[0];
         this.resourceUrn = cursor.get_string(Query.QueryColumns.RESOURCE_URN)[0];
         this.favorite = cursor.get_boolean(Query.QueryColumns.FAVORITE);
@@ -118,9 +337,7 @@ DocCommon.prototype = {
 
         this.mimeType = cursor.get_string(Query.QueryColumns.MIMETYPE)[0];
         this.rdfType = cursor.get_string(Query.QueryColumns.RDFTYPE)[0];
-        this.updateIconFromType();
-
-        this.updateTypeDescription();
+        this._updateInfoFromType();
 
         // sanitize
         if (!this.uri)
@@ -128,9 +345,9 @@ DocCommon.prototype = {
 
         let title = cursor.get_string(Query.QueryColumns.TITLE)[0];
         if (title && title != '')
-            this.title = title;
+            this.name = title;
         else
-            this.title = Gd.filename_strip_extension(
+            this.name = Gd.filename_strip_extension(
                 cursor.get_string(Query.QueryColumns.FILENAME)[0]);
 
         this._sanitizeTitle();
@@ -163,109 +380,21 @@ DocCommon.prototype = {
         this.checkEffectsAndUpdateInfo();
     },
 
-    refreshIcon: function() {
-        this.updateIconFromType();
-    },
+    _refreshCollectionIcon: function() {
+        if (!this._collectionIconWatcher) {
+            this._collectionIconWatcher = new CollectionIconWatcher(this);
 
-    _createSymbolicEmblem: function(name) {
-        let pix = Gd.create_symbolic_icon(name, Utils.getIconSize());
+            this._collectionIconWatcher.connect('icon-updated', Lang.bind(this,
+                function(watcher, pixbuf) {
+                    if (!pixbuf)
+                        return;
 
-        if (!pix)
-            pix = new Gio.ThemedIcon({ name: name });
-
-        return pix;
-    },
-
-    checkEffectsAndUpdateInfo: function() {
-        let emblemIcons = [];
-        let pixbuf = this.pixbuf;
-
-        if (this.favorite &&
-            (Global.categoryManager.getActiveCategoryId() != Categories.StockCategories.FAVORITES))
-            emblemIcons.push(this._createSymbolicEmblem('emblem-favorite'));
-        if (this.shared && (Global.categoryManager.getActiveCategoryId() != Categories.StockCategories.SHARED))
-            emblemIcons.push(this._createSymbolicEmblem('emblem-shared'));
-
-        if (emblemIcons.length > 0) {
-            let emblemedIcon = new Gio.EmblemedIcon({ gicon: this.pixbuf });
-
-            emblemIcons.forEach(
-                function(emblemIcon) {
-                    let emblem = new Gio.Emblem({ icon: emblemIcon });
-                    emblemedIcon.add_emblem(emblem);
-                });
-
-            let theme = Gtk.IconTheme.get_default();
-
-            try {
-                let iconInfo = theme.lookup_by_gicon(emblemedIcon,
-                                                     Math.max(this.pixbuf.get_width(),
-                                                              this.pixbuf.get_height()),
-                                                     Gtk.IconLookupFlags.FORCE_SIZE);
-
-                pixbuf = iconInfo.load_icon();
-            } catch (e) {
-                log('Unable to render the emblem: ' + e.toString());
-            }
+                    this.pixbuf = pixbuf;
+                    this.checkEffectsAndUpdateInfo();
+                }));
+        } else {
+            this._collectionIconWatcher.refresh();
         }
-
-        if (this.thumbnailed)
-            this.pixbuf = Gd.embed_image_in_frame(pixbuf,
-                Global.documentManager.getPixbufFrame(),
-                _THUMBNAIL_FRAME, _THUMBNAIL_FRAME,
-                _THUMBNAIL_FRAME, _THUMBNAIL_FRAME);
-        else
-            this.pixbuf = pixbuf;
-
-        this.emit('info-updated');
-    },
-
-    destroy: function() {
-        Global.settings.disconnect(this._refreshIconId);
-        Global.categoryManager.disconnect(this._categoryId);
-    },
-
-    open: function(screen, timestamp) {
-        Gtk.show_uri(screen, this.uri, timestamp);
-    },
-
-    setFavorite: function(favorite) {
-        TrackerUtils.setFavorite(this.urn, favorite, null);
-    }
-};
-Signals.addSignalMethods(DocCommon.prototype);
-
-const _FILE_ATTRIBUTES = 'thumbnail::path';
-
-function LocalDocument(cursor) {
-    this._init(cursor);
-}
-
-LocalDocument.prototype = {
-    __proto__: DocCommon.prototype,
-
-    _init: function(cursor) {
-        this._thumbPath = null;
-        this._failedThumbnailing = false;
-
-        DocCommon.prototype._init.call(this, cursor);
-
-        this.sourceName = _("Local");
-        let defaultApp = Gio.app_info_get_default_for_type(this.mimeType, true);
-        this.defaultAppName = defaultApp.get_name();
-    },
-
-    updateTypeDescription: function() {
-        this.typeDescription = Gio.content_type_get_description(this.mimeType);
-    },
-
-    _refreshThumbPath: function() {
-        this.pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(this._thumbPath,
-                                                             Utils.getIconSize(),
-                                                             Utils.getIconSize());
-        this.thumbnailed = true;
-        this.checkEffectsAndUpdateInfo();
-        return;
     },
 
     refreshIcon: function() {
@@ -274,13 +403,21 @@ LocalDocument.prototype = {
             return;
         }
 
-        if (this._failedThumbnailing) {
-            this.updateIconFromType();
+        this.updateIconFromType();
+
+        if (this.collection) {
+            this._refreshCollectionIcon();
             return;
         }
 
+        if (this._failedThumbnailing)
+            return;
+
+        if (!this._triedThumbnailing)
+            this._triedThumbnailing = true;
+
         this._file = Gio.file_new_for_uri(this.uri);
-        this._file.query_info_async(_FILE_ATTRIBUTES,
+        this._file.query_info_async(Gio.FILE_ATTRIBUTE_THUMBNAIL_PATH,
                                     0, 0, null,
                                     Lang.bind(this, this._onFileQueryInfo));
     },
@@ -341,6 +478,158 @@ LocalDocument.prototype = {
             this._failedThumbnailing = true;
     },
 
+    _refreshThumbPath: function() {
+        let thumbFile = Gio.file_new_for_path(this._thumbPath);
+
+        thumbFile.read_async(GLib.PRIORITY_DEFAULT, null, Lang.bind(this,
+            function(object, res) {
+                try {
+                    let stream = object.read_finish(res);
+                    GdkPixbuf.Pixbuf.new_from_stream_at_scale_async(stream,
+                        Utils.getIconSize(), Utils.getIconSize(),
+                        true, null, Lang.bind(this,
+                            function(object, res) {
+                                try {
+                                    this.pixbuf = GdkPixbuf.Pixbuf.new_from_stream_finish(res);
+                                    this.thumbnailed = true;
+                                    this.checkEffectsAndUpdateInfo();
+                                } catch (e) {
+                                    this._failedThumbnailing = true;
+                                }
+                            }));
+                } catch (e) {
+                    this._failedThumbnailing = true;
+                }
+            }));
+    },
+
+    _updateInfoFromType: function() {
+        if (this.rdfType.indexOf('nfo#DataContainer') != -1)
+            this.collection = true;
+
+        this.updateTypeDescription();
+    },
+
+    _createSymbolicEmblem: function(name) {
+        let pix = Gd.create_symbolic_icon(name, Utils.getIconSize());
+
+        if (!pix)
+            pix = new Gio.ThemedIcon({ name: name });
+
+        return pix;
+    },
+
+    checkEffectsAndUpdateInfo: function() {
+        let emblemIcons = [];
+        let pixbuf = this.pixbuf;
+        let activeItem;
+
+        // save the pixbuf before modifications, to use in collection icons
+        this.pristinePixbuf = pixbuf;
+
+        activeItem = Global.searchCategoryManager.getActiveItem();
+
+        if (this.favorite &&
+            (!activeItem ||
+             (activeItem.id != Searchbar.SearchCategoryStock.FAVORITES)))
+            emblemIcons.push(this._createSymbolicEmblem('emblem-favorite'));
+        if (this.shared &&
+            (!activeItem ||
+             (activeItem.id != Searchbar.SearchCategoryStock.SHARED)))
+            emblemIcons.push(this._createSymbolicEmblem('emblem-shared'));
+
+        if (emblemIcons.length > 0) {
+            let emblemedIcon = new Gio.EmblemedIcon({ gicon: this.pixbuf });
+
+            emblemIcons.forEach(
+                function(emblemIcon) {
+                    let emblem = new Gio.Emblem({ icon: emblemIcon });
+                    emblemedIcon.add_emblem(emblem);
+                });
+
+            let theme = Gtk.IconTheme.get_default();
+
+            try {
+                let iconInfo = theme.lookup_by_gicon(emblemedIcon,
+                                                     Math.max(this.pixbuf.get_width(),
+                                                              this.pixbuf.get_height()),
+                                                     Gtk.IconLookupFlags.FORCE_SIZE);
+
+                pixbuf = iconInfo.load_icon();
+            } catch (e) {
+                log('Unable to render the emblem: ' + e.toString());
+            }
+        }
+
+        if (this.thumbnailed) {
+            let [ slice, border ] = Utils.getThumbnailFrameBorder();
+            this.pixbuf = Gd.embed_image_in_frame(pixbuf,
+                Path.ICONS_DIR + 'thumbnail-frame.png',
+                slice, border);
+        } else {
+            this.pixbuf = pixbuf;
+        }
+
+        this.emit('info-updated');
+    },
+
+    destroy: function() {
+        if (this._collectionIconWatcher) {
+            this._collectionIconWatcher.destroy();
+            this._collectionIconWatcher = null;
+        }
+
+        Global.settings.disconnect(this._refreshIconId);
+        Global.searchCategoryManager.disconnect(this._filterId);
+    },
+
+    open: function(screen, timestamp) {
+        Gtk.show_uri(screen, this.uri, timestamp);
+    },
+
+    setFavorite: function(favorite) {
+        TrackerUtils.setFavorite(this.id, favorite, null);
+    },
+
+    getWhere: function() {
+        let retval = '';
+
+        if (this.collection)
+            retval = '{ ?urn nie:isPartOf <' + this.id + '> }';
+
+        return retval;
+    }
+};
+Signals.addSignalMethods(DocCommon.prototype);
+
+function LocalDocument(cursor) {
+    this._init(cursor);
+}
+
+LocalDocument.prototype = {
+    __proto__: DocCommon.prototype,
+
+    _init: function(cursor) {
+        this._failedThumbnailing = false;
+        this._triedThumbnailing = false;
+
+        DocCommon.prototype._init.call(this, cursor);
+
+        this.sourceName = _("Local");
+
+        let defaultApp = null;
+        if (this.mimeType)
+            defaultApp = Gio.app_info_get_default_for_type(this.mimeType, true);
+
+        if (defaultApp)
+            this.defaultAppName = defaultApp.get_name();
+    },
+
+    updateTypeDescription: function() {
+        if (this.mimeType)
+            this.typeDescription = Gio.content_type_get_description(this.mimeType);
+    },
+
     loadPreview: function(cancellable, callback) {
         Gd.pdf_loader_load_uri_async(this.uri, cancellable, Lang.bind(this,
             function(source, res) {
@@ -351,6 +640,24 @@ LocalDocument.prototype = {
                     Global.errorHandler.addLoadError(this, e);
                 }
             }));
+    },
+
+    canTrash: function() {
+        return this.collection;
+    },
+
+    trash: function() {
+        if (this.collection) {
+            let job = new DeleteItemJob(this.id);
+            job.run(Lang.bind(this,
+                function() {
+                    // FIXME: this should be done automatically when tracker
+                    // will support change notifications for collections
+                    Global.documentManager.getModel().documentRemoved(this);
+                    Global.documentManager.removeItemById(this.id);
+                    Global.collectionManager.removeItemById(this.id);
+                }));
+        }
     }
 };
 
@@ -365,16 +672,18 @@ GoogleDocument.prototype = {
     __proto__: DocCommon.prototype,
 
     _init: function(cursor) {
+        this._triedThumbnailing = true;
+        this._failedThumbnailing = true;
+
         DocCommon.prototype._init.call(this, cursor);
 
         // overridden
-        this.identifier = cursor.get_string(Query.QueryColumns.IDENTIFIER)[0];
         this.defaultAppName = _("Google Docs");
         this.sourceName = _("Google");
     },
 
     _createGDataEntry: function(cancellable, callback) {
-        let source = Global.sourceManager.getSourceByUrn(this.resourceUrn);
+        let source = Global.sourceManager.getItemById(this.resourceUrn);
 
         let authorizer = new Gd.GDataGoaAuthorizer({ goa_object: source.object });
         let service = new GData.DocumentsService({ authorizer: authorizer });
@@ -442,6 +751,8 @@ GoogleDocument.prototype = {
             description = _("Spreadsheet");
         else if (this.rdfType.indexOf('nfo#Presentation') != -1)
             description = _("Presentation");
+        else if (this.rdfType.indexOf('nfo#DataContainer') != -1)
+            description = _("Collection");
         else
             description = _("Document");
 
@@ -460,7 +771,7 @@ GoogleDocument.prototype = {
         this._createGDataEntry(null, Lang.bind(this,
             function(entry, service, exception) {
                 if (!entry) {
-                    log('Unable to call setFavorite on ' + this.title + ': ' + exception.toString());
+                    log('Unable to call setFavorite on ' + this.name + ': ' + exception.toString());
                     return;
                 }
 
@@ -488,10 +799,14 @@ GoogleDocument.prototype = {
                              try {
                                  service.update_entry_finish(res);
                              } catch (e) {
-                                 log('Unable to call setFavorite on ' + this.title + ': ' + e.toString());
+                                 log('Unable to call setFavorite on ' + this.name + ': ' + e.toString());
                              }
                          }));
             }));
+    },
+
+    canTrash: function() {
+        return false;
     }
 };
 
@@ -500,16 +815,15 @@ function DocumentManager() {
 }
 
 DocumentManager.prototype = {
+    __proto__: Manager.BaseManager.prototype,
+
     _init: function() {
-        this._docs = {};
-        this._activeDocument = null;
+        Manager.BaseManager.prototype._init.call(this);
 
         this._model = new DocumentModel();
 
         Global.changeMonitor.connect('changes-pending',
                                      Lang.bind(this, this._onChangesPending));
-
-        this._pixbufFrame = GdkPixbuf.Pixbuf.new_from_file(Path.ICONS_DIR + 'thumbnail-frame.png');
     },
 
     _onChangesPending: function(monitor, changes) {
@@ -517,46 +831,36 @@ DocumentManager.prototype = {
             let changeEvent = changes[idx];
 
             if (changeEvent.type == ChangeMonitor.ChangeEventType.CHANGED) {
-                let doc = this.lookupDocument(changeEvent.urn);
+                let doc = this.getItemById(changeEvent.urn);
 
                 if (doc)
                     doc.refresh();
             } else if (changeEvent.type == ChangeMonitor.ChangeEventType.CREATED) {
                 this._onDocumentCreated(changeEvent.urn);
             } else if (changeEvent.type == ChangeMonitor.ChangeEventType.DELETED) {
-                let doc = this.lookupDocument(changeEvent.urn);
+                let doc = this.getItemById(changeEvent.urn);
 
                 if (doc) {
                     this._model.documentRemoved(doc);
 
                     doc.destroy();
-                    delete this._docs[changeEvent.urn];
+                    this.removeItemById(changeEvent.urn);
+
+                    if (doc.collection)
+                        Global.collectionManager.removeItemById(changeEvent.urn);
                 }
             }
         }
     },
 
     _onDocumentCreated: function(urn) {
-        let sparql = Global.queryBuilder.buildSingleQuery(urn);
-
-        Global.connection.query_async(sparql, null, Lang.bind(this,
-            function(object, res) {
-                let cursor = null;
-
-                try {
-                    cursor = object.query_finish(res);
-                    cursor.next_async(null, Lang.bind(this,
-                        function(object, res) {
-                            let valid = object.next_finish(res);
-                            if (valid)
-                                this.addDocument(object);
-
-                            cursor.close();
-                        }));
-                } catch (e) {
-                    log('Unable to add new document: ' + e.toString());
+        let job = new SingleItemJob(urn);
+        job.run(Query.QueryFlags.NONE, Lang.bind(this,
+            function(cursor) {
+                if (!cursor)
                     return;
-                }
+
+                this.addDocumentFromCursor(cursor);
             }));
     },
 
@@ -565,11 +869,7 @@ DocumentManager.prototype = {
                 (identifier.indexOf('https://docs.google.com') != -1));
     },
 
-    getPixbufFrame: function() {
-        return this._pixbufFrame;
-    },
-
-    addDocument: function(cursor) {
+    createDocumentFromCursor: function(cursor) {
         let identifier = cursor.get_string(Query.QueryColumns.IDENTIFIER)[0];
         let doc;
 
@@ -578,47 +878,42 @@ DocumentManager.prototype = {
         else
             doc = new LocalDocument(cursor);
 
-        this._docs[doc.urn] = doc;
+        return doc;
+    },
+
+    addDocumentFromCursor: function(cursor) {
+        let doc = this.createDocumentFromCursor(cursor);
+        this.addItem(doc);
         this._model.documentAdded(doc);
+
+        if (doc.collection)
+            Global.collectionManager.addItem(doc);
     },
 
     clear: function() {
-        for (idx in this._docs) {
-            this._docs[idx].destroy();
+        let items = this.getItems();
+        for (idx in items) {
+            items[idx].destroy();
         };
-        this._docs = {};
+
+        Manager.BaseManager.prototype.clear.call(this);
         this._model.clear();
     },
 
-    getDocuments: function() {
-        return this._docs;
-    },
+    setActiveItem: function(doc) {
+        if (Manager.BaseManager.prototype.setActiveItem.call(this, doc)) {
 
-    lookupDocument: function(urn) {
-        let document = null;
-
-        if (this._docs[urn])
-            document = this._docs[urn];
-
-        return document;
-    },
-
-    setActiveDocument: function(doc) {
-        if (doc == this._activeDocument)
-            return;
-
-        this._activeDocument = doc;
-    },
-
-    getActiveDocument: function() {
-        return this._activeDocument;
+            if (doc && !doc.collection) {
+                let recentManager = Gtk.RecentManager.get_default();
+                recentManager.add_item(doc.uri);
+            }
+        }
     },
 
     getModel: function() {
         return this._model;
     }
 };
-Signals.addSignalMethods(DocumentManager.prototype);
 
 const ModelColumns = {
     URN: 0,
@@ -645,20 +940,26 @@ DocumentModel.prototype = {
 
     documentAdded: function(doc) {
         let iter = this.model.append();
-        let treePath = this.model.get_path(iter);
 
         Gd.store_set(this.model, iter,
-                     doc.urn,
-                     doc.title, doc.author,
+                     doc.id,
+                     doc.name, doc.author,
                      doc.pixbuf, doc.mtime);
 
+        let treePath = this.model.get_path(iter);
+        let treeRowRef = Gtk.TreeRowReference.new(this.model, treePath);
+
         doc.connect('info-updated', Lang.bind(this,
             function() {
-                let objectIter = this.model.get_iter(treePath)[1];
+                let objectPath = treeRowRef.get_path();
+                if (!objectPath)
+                    return;
+
+                let objectIter = this.model.get_iter(objectPath)[1];
                 if (objectIter)
                     Gd.store_set(this.model, iter,
-                                 doc.urn,
-                                 doc.title, doc.author,
+                                 doc.id,
+                                 doc.name, doc.author,
                                  doc.pixbuf, doc.mtime);
             }));
     },
@@ -668,7 +969,7 @@ DocumentModel.prototype = {
             function(model, path, iter) {
                 let urn = model.get_value(iter, ModelColumns.URN);
 
-                if (urn == doc.urn) {
+                if (urn == doc.id) {
                     this.model.remove(iter);
                     return true;
                 }
