@@ -20,13 +20,21 @@
  */
 
 let application = null;
-let categoryManager = null;
+let collectionManager = null;
 let connection = null;
+let connectionQueue = null;
 let documentManager = null;
 let errorHandler = null;
+let goaClient = null;
+let modeController = null;
 let offsetController = null;
 let queryBuilder = null;
+let searchCategoryManager = null;
+let searchController = null;
+let searchMatchManager = null;
+let searchTypeManager = null;
 let selectionController = null;
 let settings = null;
+let stage = null;
 let sourceManager = null;
 let trackerController = null;
